#! /bin/sh /usr/share/dpatch/dpatch-run
## 40_printer_margins.dpatch by Alexey Antipov (1a_antipov@mail.ru)
##
## All lines beginning with `## DP:' are a description of the patch.
## DP: Allow user to specify printer margins in printer settings and
## DP: printing dialog

@DPATCH@
--- ./src/utils.c	(revision 5530)
+++ ./src/utils.c	(working copy)
@@ -840,7 +840,36 @@
 	return tmp;
 }
 
+/**
+ *  Wraps g_key_file_get_double() to add a default value argument.
+ *
+ *  @param config A GKeyFile object.
+ *  @param section The group name to look in for the key.
+ *  @param key The key to find.
+ *  @param default_value The default value which will be returned when @a section or @a key
+ *         don't exist.
+ *
+ *  @return The value associated with @a key as an double, or the given default value if the value
+ *          could not be retrieved.
+ **/
+gdouble utils_get_setting_double(GKeyFile *config, const gchar *section, const gchar *key,
+							   const gdouble default_value)
+{
+	gdouble tmp;
+	GError *error = NULL;
 
+	if (G_UNLIKELY(config == NULL))
+		return default_value;
+
+	tmp = g_key_file_get_double(config, section, key, &error);
+	if (G_UNLIKELY(error))
+	{
+		g_error_free(error);
+		return default_value;
+	}
+	return tmp;
+}
+
 /**
  *  Wraps g_key_file_get_boolean() to add a default value argument.
  *
@@ -1019,6 +1048,47 @@
 	else
 		return 0;
 }
+/*
+ * utils_combo_box_get_units() obtains selected GtkUnit value from 
+ * a combobox, which contains exactly 4 values: px, pt, inch and mm
+ */
+GtkUnit utils_combo_box_get_units(GtkComboBox *combobox)
+{
+	switch (gtk_combo_box_get_active(combobox)) {
+		case 0:
+			return GTK_UNIT_PIXEL;
+		case 1:
+			return GTK_UNIT_POINTS;
+		case 2:
+			return GTK_UNIT_INCH;
+		case 3:
+			return GTK_UNIT_MM;
+		default:
+			return 0;
+	}
+}
+/*
+ * utils_combo_box_set_units() selects a value in a combobox depending
+ * on Gtkunit value supplied. The combobox must contain exactly 
+ * 4 values: px, pt, inch and mm
+ */
+void utils_combo_box_set_units(GtkComboBox *combobox, GtkUnit unit)
+{
+	switch (unit) {
+		case GTK_UNIT_PIXEL:
+			gtk_combo_box_set_active(combobox, 0);
+			break;
+		case GTK_UNIT_POINTS:
+			gtk_combo_box_set_active(combobox, 1);
+			break;
+		case GTK_UNIT_INCH:
+			gtk_combo_box_set_active(combobox, 2);
+			break;
+		case GTK_UNIT_MM:
+			gtk_combo_box_set_active(combobox, 3);
+			break;
+	}
+}
 
 
 /* utils_strtod() converts a string containing a hex colour ("0x00ff00") into an integer.
--- ./src/interface.c	(revision 5530)
+++ ./src/interface.c	(working copy)
@@ -2888,6 +2888,28 @@
   GtkWidget *hbox10;
   GtkWidget *label203;
   GtkWidget *entry_print_dateformat;
+  GtkWidget *check_custom_margins;
+  GtkWidget *frame1;
+  GtkWidget *alignment1;
+  GtkWidget *hbox1;
+  GtkWidget *table3;
+  GtkWidget *label1;
+  GtkWidget *label2;
+  GtkObject *spin_margin_left_adj;
+  GtkWidget *spin_margin_left;
+  GtkWidget *combo_margin_left_units;
+  GtkObject *spin_margin_right_adj;
+  GtkWidget *spin_margin_right;
+  GtkWidget *combo_margin_right_units;
+  GtkWidget *table4;
+  GtkWidget *label3;
+  GtkWidget *label4;
+  GtkObject *spin_margin_top_adj;
+  GtkWidget *spin_margin_top;
+  GtkWidget *combo_margin_top_units;
+  GtkObject *spin_margin_bottom_adj;
+  GtkWidget *spin_margin_bottom;
+  GtkWidget *combo_margin_bottom_units;
   GtkWidget *radio_print_gtk;
   GtkWidget *label243;
   GtkWidget *label201;
@@ -4966,6 +4988,132 @@
   gtk_tooltips_set_tip (tooltips, entry_print_dateformat, _("Specify a format for the date and time stamp which is added to the page header on each page. You can use any conversion specifiers which can be used with the ANSI C strftime function."), NULL);
   gtk_entry_set_invisible_char (GTK_ENTRY (entry_print_dateformat), 9679);
 
+  check_custom_margins = gtk_check_button_new_with_mnemonic (_("Use custom printer margins"));
+  gtk_widget_show (check_custom_margins);
+  gtk_box_pack_start (GTK_BOX (vbox29), check_custom_margins, FALSE, FALSE, 0);
+  gtk_tooltips_set_tip (tooltips, check_custom_margins, _("Use custom printer margins, overriding printer settings"), NULL);
+
+  frame1 = gtk_frame_new (NULL);
+  gtk_widget_show (frame1);
+  gtk_box_pack_start (GTK_BOX (vbox29), frame1, FALSE, FALSE, 0);
+  gtk_frame_set_label_align (GTK_FRAME (frame1), 0, 0);
+  gtk_frame_set_shadow_type(GTK_FRAME(frame1), GTK_SHADOW_NONE);
+
+  alignment1 = gtk_alignment_new (0, 0.5, 1, 1);
+  gtk_widget_show (alignment1);
+  gtk_container_add (GTK_CONTAINER (frame1), alignment1);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment1), 0, 0, 12, 0);
+
+  hbox1 = gtk_hbox_new (FALSE, 10);
+  gtk_widget_show (hbox1);
+  gtk_container_add (GTK_CONTAINER (alignment1), hbox1);
+
+  table3 = gtk_table_new (2, 3, FALSE);
+  gtk_widget_show (table3);
+  gtk_box_pack_start (GTK_BOX (hbox1), table3, FALSE, FALSE, 0);
+  gtk_table_set_col_spacings (GTK_TABLE (table3), 5);
+
+  label1 = gtk_label_new (_("Left:"));
+  gtk_widget_show (label1);
+  gtk_table_attach (GTK_TABLE (table3), label1, 0, 1, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label1), 1, 0.5);
+
+  label2 = gtk_label_new (_("Right:"));
+  gtk_widget_show (label2);
+  gtk_table_attach (GTK_TABLE (table3), label2, 0, 1, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label2), 1, 0.5);
+
+  spin_margin_left_adj = gtk_adjustment_new (10, 0, 100, 1, 10, 10);
+  spin_margin_left = gtk_spin_button_new (GTK_ADJUSTMENT (spin_margin_left_adj), 1, 0);
+  gtk_widget_show (spin_margin_left);
+  gtk_table_attach (GTK_TABLE (table3), spin_margin_left, 1, 2, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+
+  combo_margin_left_units = gtk_combo_box_new_text ();
+  gtk_widget_show (combo_margin_left_units);
+  gtk_table_attach (GTK_TABLE (table3), combo_margin_left_units, 2, 3, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_left_units), _("px"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_left_units), _("pt"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_left_units), _("inch"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_left_units), _("mm"));
+
+  spin_margin_right_adj = gtk_adjustment_new (10, 0, 100, 1, 10, 10);
+  spin_margin_right = gtk_spin_button_new (GTK_ADJUSTMENT (spin_margin_right_adj), 1, 0);
+  gtk_widget_show (spin_margin_right);
+  gtk_table_attach (GTK_TABLE (table3), spin_margin_right, 1, 2, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+
+  combo_margin_right_units = gtk_combo_box_new_text ();
+  gtk_widget_show (combo_margin_right_units);
+  gtk_table_attach (GTK_TABLE (table3), combo_margin_right_units, 2, 3, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_right_units), _("px"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_right_units), _("pt"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_right_units), _("inch"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_right_units), _("mm"));
+
+  table4 = gtk_table_new (2, 3, FALSE);
+  gtk_widget_show (table4);
+  gtk_box_pack_start (GTK_BOX (hbox1), table4, FALSE, FALSE, 0);
+  gtk_table_set_col_spacings (GTK_TABLE (table4), 5);
+
+  label3 = gtk_label_new (_("Top:"));
+  gtk_widget_show (label3);
+  gtk_table_attach (GTK_TABLE (table4), label3, 0, 1, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label3), 1, 0.5);
+
+  label4 = gtk_label_new (_("Bottom:"));
+  gtk_widget_show (label4);
+  gtk_table_attach (GTK_TABLE (table4), label4, 0, 1, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label4), 1, 0.5);
+
+  spin_margin_top_adj = gtk_adjustment_new (10, 0, 100, 1, 10, 10);
+  spin_margin_top = gtk_spin_button_new (GTK_ADJUSTMENT (spin_margin_top_adj), 1, 0);
+  gtk_widget_show (spin_margin_top);
+  gtk_table_attach (GTK_TABLE (table4), spin_margin_top, 1, 2, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+
+  combo_margin_top_units = gtk_combo_box_new_text ();
+  gtk_widget_show (combo_margin_top_units);
+  gtk_table_attach (GTK_TABLE (table4), combo_margin_top_units, 2, 3, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_top_units), _("px"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_top_units), _("pt"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_top_units), _("inch"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_top_units), _("mm"));
+
+  spin_margin_bottom_adj = gtk_adjustment_new (10, 0, 100, 1, 10, 10);
+  spin_margin_bottom = gtk_spin_button_new (GTK_ADJUSTMENT (spin_margin_bottom_adj), 1, 0);
+  gtk_widget_show (spin_margin_bottom);
+  gtk_table_attach (GTK_TABLE (table4), spin_margin_bottom, 1, 2, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+
+  combo_margin_bottom_units = gtk_combo_box_new_text ();
+  gtk_widget_show (combo_margin_bottom_units);
+  gtk_table_attach (GTK_TABLE (table4), combo_margin_bottom_units, 2, 3, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_bottom_units), _("px"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_bottom_units), _("pt"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_bottom_units), _("inch"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo_margin_bottom_units), _("mm"));
+
   radio_print_gtk = gtk_radio_button_new_with_mnemonic (NULL, _("Use native GTK printing"));
   gtk_widget_show (radio_print_gtk);
   gtk_frame_set_label_widget (GTK_FRAME (frame31), radio_print_gtk);
@@ -5388,6 +5536,24 @@
   GLADE_HOOKUP_OBJECT (prefs_dialog, hbox10, "hbox10");
   GLADE_HOOKUP_OBJECT (prefs_dialog, label203, "label203");
   GLADE_HOOKUP_OBJECT (prefs_dialog, entry_print_dateformat, "entry_print_dateformat");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, check_custom_margins, "check_custom_margins");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, frame1, "frame1");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, alignment1, "alignment1");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, hbox1, "hbox1");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, table3, "table3");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, label1, "label1");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, label2, "label2");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, spin_margin_left, "spin_margin_left");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, combo_margin_left_units, "combo_margin_left_units");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, spin_margin_right, "spin_margin_right");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, combo_margin_right_units, "combo_margin_right_units");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, table4, "table4");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, label3, "label3");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, label4, "label4");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, spin_margin_top, "spin_margin_top");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, combo_margin_top_units, "combo_margin_top_units");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, spin_margin_bottom, "spin_margin_bottom");
+  GLADE_HOOKUP_OBJECT (prefs_dialog, combo_margin_bottom_units, "combo_margin_bottom_units");
   GLADE_HOOKUP_OBJECT (prefs_dialog, radio_print_gtk, "radio_print_gtk");
   GLADE_HOOKUP_OBJECT (prefs_dialog, label243, "label243");
   GLADE_HOOKUP_OBJECT (prefs_dialog, label201, "label201");
--- ./src/utils.h	(revision 5530)
+++ ./src/utils.h	(working copy)
@@ -184,6 +184,8 @@
 
 gint utils_get_setting_integer(GKeyFile *config, const gchar *section, const gchar *key, const gint default_value);
 
+gdouble utils_get_setting_double(GKeyFile *config, const gchar *section, const gchar *key, const gdouble default_value);
+
 gchar *utils_get_setting_string(GKeyFile *config, const gchar *section, const gchar *key, const gchar *default_value);
 
 gchar *utils_get_hex_from_color(GdkColor *color);
@@ -199,6 +201,10 @@
 gchar *utils_make_human_readable_str(guint64 size, gulong block_size,
 									 gulong display_unit);
 
+void utils_combo_box_set_units(GtkComboBox *combobox, GtkUnit unit);
+
+GtkUnit utils_combo_box_get_units(GtkComboBox *combobox);
+
 gint utils_strtod(const gchar *source, gchar **end, gboolean with_route);
 
 gchar *utils_get_current_time_string(void);
--- ./src/printing.c	(revision 5530)
+++ ./src/printing.c	(working copy)
@@ -99,6 +99,15 @@
 	GtkWidget *check_print_pageheader;
 	GtkWidget *check_print_basename;
 	GtkWidget *entry_print_dateformat;
+	GtkWidget *check_custom_margins;
+	GtkWidget *spin_margin_left;
+	GtkWidget *combo_margin_left_units;
+	GtkWidget *spin_margin_right;
+	GtkWidget *combo_margin_right_units;
+	GtkWidget *spin_margin_top;
+	GtkWidget *combo_margin_top_units;
+	GtkWidget *spin_margin_bottom;
+	GtkWidget *combo_margin_bottom_units;
 } PrintWidgets;
 
 
@@ -350,6 +359,18 @@
 	g_free(printing_prefs.page_header_datefmt);
 	printing_prefs.page_header_datefmt =
 		g_strdup(gtk_entry_get_text(GTK_ENTRY(w->entry_print_dateformat)));
+		
+	printing_prefs.margin_left = gtk_spin_button_get_value(GTK_SPIN_BUTTON(w->spin_margin_left));
+	printing_prefs.margin_left_units = utils_combo_box_get_units(GTK_COMBO_BOX(w->combo_margin_left_units));
+	
+	printing_prefs.margin_right = gtk_spin_button_get_value(GTK_SPIN_BUTTON(w->spin_margin_right));
+	printing_prefs.margin_right_units = utils_combo_box_get_units(GTK_COMBO_BOX(w->combo_margin_right_units));
+	
+	printing_prefs.margin_top = gtk_spin_button_get_value(GTK_SPIN_BUTTON(w->spin_margin_top));
+	printing_prefs.margin_top_units = utils_combo_box_get_units(GTK_COMBO_BOX(w->combo_margin_top_units));
+	
+	printing_prefs.margin_bottom = gtk_spin_button_get_value(GTK_SPIN_BUTTON(w->spin_margin_bottom));
+	printing_prefs.margin_bottom_units = utils_combo_box_get_units(GTK_COMBO_BOX(w->combo_margin_bottom_units));
 }
 
 
@@ -362,7 +383,25 @@
 	gtk_widget_set_sensitive(w->entry_print_dateformat, sens);
 }
 
+static void on_custom_printer_margins_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	gboolean sens = gtk_toggle_button_get_active(togglebutton);
+	PrintWidgets *w = user_data;
+		
+	gtk_widget_set_sensitive(w->spin_margin_left, sens);
+	gtk_widget_set_sensitive(w->combo_margin_left_units, sens);
+	
+	gtk_widget_set_sensitive(w->spin_margin_right, sens);
+	gtk_widget_set_sensitive(w->combo_margin_right_units, sens);
+	
+	gtk_widget_set_sensitive(w->spin_margin_top, sens);
+	gtk_widget_set_sensitive(w->combo_margin_top_units, sens);
+	
+	gtk_widget_set_sensitive(w->spin_margin_bottom, sens);
+	gtk_widget_set_sensitive(w->combo_margin_bottom_units, sens);
+}
 
+
 static GtkWidget *create_custom_widget(GtkPrintOperation *operation, gpointer user_data)
 {	/* copied from interface.c */
 	GtkWidget *page;
@@ -371,6 +410,20 @@
 	GtkWidget *vbox30;
 	GtkWidget *hbox10;
 	GtkWidget *label203;
+	GtkWidget *frame1;
+  GtkWidget *alignment1;
+  GtkWidget *hbox1;
+  GtkWidget *table3;
+  GtkWidget *label1;
+  GtkWidget *label2;
+  GtkObject *spin_margin_left_adj;
+  GtkObject *spin_margin_right_adj;
+  GtkWidget *table4;
+  GtkWidget *label3;
+  GtkWidget *label4;
+  GtkObject *spin_margin_top_adj;
+  GtkObject *spin_margin_bottom_adj;
+  
 	PrintWidgets *w = user_data;
 
 	gtk_print_operation_set_custom_tab_label(operation, _("Document Setup"));
@@ -424,6 +477,145 @@
 	gtk_entry_set_text(GTK_ENTRY(w->entry_print_dateformat), printing_prefs.page_header_datefmt);
 
 	on_page_header_toggled(GTK_TOGGLE_BUTTON(w->check_print_pageheader), w);
+	
+	w->check_custom_margins = gtk_check_button_new_with_mnemonic (_("Use custom printer margins"));
+  gtk_widget_show (w->check_custom_margins);
+  gtk_box_pack_start (GTK_BOX (page), w->check_custom_margins, FALSE, FALSE, 0);
+  ui_widget_set_tooltip_text(w->check_custom_margins, _("Use custom printer margins, overriding printer settings"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w->check_custom_margins), printing_prefs.custom_printer_margins);
+	g_signal_connect(w->check_custom_margins, "toggled", G_CALLBACK(on_custom_printer_margins_toggled), w);
+	
+  frame1 = gtk_frame_new (NULL);
+  gtk_widget_show (frame1);
+  gtk_box_pack_start (GTK_BOX (page), frame1, FALSE, FALSE, 0);
+  gtk_frame_set_label_align (GTK_FRAME (frame1), 0, 0);
+	gtk_frame_set_shadow_type(GTK_FRAME(frame1), GTK_SHADOW_NONE);
+
+  alignment1 = gtk_alignment_new (0, 0.5, 1, 1);
+  gtk_widget_show (alignment1);
+  gtk_container_add (GTK_CONTAINER (frame1), alignment1);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment1), 0, 0, 12, 0);
+
+  hbox1 = gtk_hbox_new (FALSE, 10);
+  gtk_widget_show (hbox1);
+  gtk_container_add (GTK_CONTAINER (alignment1), hbox1);
+
+  table3 = gtk_table_new (2, 3, FALSE);
+  gtk_widget_show (table3);
+  gtk_box_pack_start (GTK_BOX (hbox1), table3, FALSE, FALSE, 0);
+  gtk_table_set_col_spacings (GTK_TABLE (table3), 5);
+
+  label1 = gtk_label_new (_("Left:"));
+  gtk_widget_show (label1);
+  gtk_table_attach (GTK_TABLE (table3), label1, 0, 1, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label1), 1, 0.5);
+
+  label2 = gtk_label_new (_("Right:"));
+  gtk_widget_show (label2);
+  gtk_table_attach (GTK_TABLE (table3), label2, 0, 1, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label2), 1, 0.5);
+
+  spin_margin_left_adj = gtk_adjustment_new (10, 0, 100, 1, 10, 10);
+  w->spin_margin_left = gtk_spin_button_new (GTK_ADJUSTMENT (spin_margin_left_adj), 1, 0);
+  gtk_widget_show (w->spin_margin_left);
+  gtk_table_attach (GTK_TABLE (table3), w->spin_margin_left, 1, 2, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w->spin_margin_left), printing_prefs.margin_left);
+	
+  w->combo_margin_left_units = gtk_combo_box_new_text ();
+  gtk_widget_show (w->combo_margin_left_units);
+  gtk_table_attach (GTK_TABLE (table3), w->combo_margin_left_units, 2, 3, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_left_units), _("px"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_left_units), _("pt"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_left_units), _("inch"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_left_units), _("mm"));
+	utils_combo_box_set_units(GTK_COMBO_BOX (w->combo_margin_left_units), printing_prefs.margin_left_units);
+	
+  spin_margin_right_adj = gtk_adjustment_new (10, 0, 100, 1, 10, 10);
+  w->spin_margin_right = gtk_spin_button_new (GTK_ADJUSTMENT (spin_margin_right_adj), 1, 0);
+  gtk_widget_show (w->spin_margin_right);
+  gtk_table_attach (GTK_TABLE (table3), w->spin_margin_right, 1, 2, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w->spin_margin_right), printing_prefs.margin_right);
+	
+  w->combo_margin_right_units = gtk_combo_box_new_text ();
+  gtk_widget_show (w->combo_margin_right_units);
+  gtk_table_attach (GTK_TABLE (table3), w->combo_margin_right_units, 2, 3, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_right_units), _("px"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_right_units), _("pt"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_right_units), _("inch"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_right_units), _("mm"));
+	utils_combo_box_set_units(GTK_COMBO_BOX (w->combo_margin_right_units), printing_prefs.margin_right_units);
+
+  table4 = gtk_table_new (2, 3, FALSE);
+  gtk_widget_show (table4);
+  gtk_box_pack_start (GTK_BOX (hbox1), table4, FALSE, FALSE, 0);
+  gtk_table_set_col_spacings (GTK_TABLE (table4), 5);
+
+  label3 = gtk_label_new (_("Top:"));
+  gtk_widget_show (label3);
+  gtk_table_attach (GTK_TABLE (table4), label3, 0, 1, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label3), 1, 0.5);
+
+  label4 = gtk_label_new (_("Bottom:"));
+  gtk_widget_show (label4);
+  gtk_table_attach (GTK_TABLE (table4), label4, 0, 1, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label4), 1, 0.5);
+
+  spin_margin_top_adj = gtk_adjustment_new (10, 0, 100, 1, 10, 10);
+  w->spin_margin_top = gtk_spin_button_new (GTK_ADJUSTMENT (spin_margin_top_adj), 1, 0);
+  gtk_widget_show (w->spin_margin_top);
+  gtk_table_attach (GTK_TABLE (table4), w->spin_margin_top, 1, 2, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w->spin_margin_top), printing_prefs.margin_top);
+
+  w->combo_margin_top_units = gtk_combo_box_new_text ();
+  gtk_widget_show (w->combo_margin_top_units);
+  gtk_table_attach (GTK_TABLE (table4), w->combo_margin_top_units, 2, 3, 0, 1,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_top_units), _("px"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_top_units), _("pt"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_top_units), _("inch"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_top_units), _("mm"));
+	utils_combo_box_set_units(GTK_COMBO_BOX (w->combo_margin_top_units), printing_prefs.margin_top_units);
+
+  spin_margin_bottom_adj = gtk_adjustment_new (10, 0, 100, 1, 10, 10);
+  w->spin_margin_bottom = gtk_spin_button_new (GTK_ADJUSTMENT (spin_margin_bottom_adj), 1, 0);
+  gtk_widget_show (w->spin_margin_bottom);
+  gtk_table_attach (GTK_TABLE (table4), w->spin_margin_bottom, 1, 2, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w->spin_margin_bottom), printing_prefs.margin_bottom);
+  
+  w->combo_margin_bottom_units = gtk_combo_box_new_text ();
+  gtk_widget_show (w->combo_margin_bottom_units);
+  gtk_table_attach (GTK_TABLE (table4), w->combo_margin_bottom_units, 2, 3, 1, 2,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_bottom_units), _("px"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_bottom_units), _("pt"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_bottom_units), _("inch"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (w->combo_margin_bottom_units), _("mm"));
+	utils_combo_box_set_units(GTK_COMBO_BOX (w->combo_margin_bottom_units), printing_prefs.margin_bottom_units);
+	
+	on_custom_printer_margins_toggled(GTK_TOGGLE_BUTTON(w->check_custom_margins), w);
+	
 	gtk_widget_show_all(page);
 	return page;
 }
@@ -786,6 +978,16 @@
 		msgwin_status_add(_("Document %s was sent to the printing subsystem."), filename);
 }
 
+static void request_page_setup(GtkPrintOperation *operation, GtkPrintContext *context, gint page_nr, GtkPageSetup *setup, gpointer user_data) 
+{
+	if (!printing_prefs.custom_printer_margins)
+		return;
+	
+	gtk_page_setup_set_top_margin(setup, printing_prefs.margin_top, printing_prefs.margin_top_units);
+	gtk_page_setup_set_bottom_margin(setup, printing_prefs.margin_bottom, printing_prefs.margin_bottom_units);
+	gtk_page_setup_set_left_margin(setup, printing_prefs.margin_left, printing_prefs.margin_left_units);
+	gtk_page_setup_set_right_margin(setup, printing_prefs.margin_right, printing_prefs.margin_right_units);
+}
 
 static void printing_print_gtk(GeanyDocument *doc)
 {
@@ -817,6 +1019,7 @@
 	g_signal_connect(op, "status-changed", G_CALLBACK(status_changed), doc->file_name);
 	g_signal_connect(op, "create-custom-widget", G_CALLBACK(create_custom_widget), widgets);
 	g_signal_connect(op, "custom-widget-apply", G_CALLBACK(custom_widget_apply), widgets);
+	g_signal_connect(op, "request-page-setup", G_CALLBACK(request_page_setup), dinfo);
 
 	if (settings != NULL)
 		gtk_print_operation_set_print_settings(op, settings);
--- ./src/printing.h	(revision 5530)
+++ ./src/printing.h	(working copy)
@@ -36,6 +36,16 @@
 	gboolean page_header_basename;
 	gchar *page_header_datefmt;
 	gchar *external_print_cmd;
+	/* Margins */
+	gboolean custom_printer_margins;
+	gdouble margin_left;
+	GtkUnit margin_left_units;
+	gdouble margin_right;
+	GtkUnit margin_right_units;
+	gdouble margin_top;
+	GtkUnit margin_top_units;
+	gdouble margin_bottom;
+	GtkUnit margin_bottom_units;
 } PrintingPrefs;
 
 extern PrintingPrefs printing_prefs;
--- ./src/prefs.c	(revision 5530)
+++ ./src/prefs.c	(working copy)
@@ -92,6 +92,7 @@
 static void on_sidebar_visible_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_prefs_print_radio_button_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_prefs_print_page_header_toggled(GtkToggleButton *togglebutton, gpointer user_data);
+static void on_prefs_custom_printer_margins_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 
 
 typedef enum PrefCallbackAction
@@ -678,6 +677,34 @@
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), printing_prefs.print_page_header);
 	on_prefs_print_page_header_toggled(GTK_TOGGLE_BUTTON(widget), NULL);
 
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_custom_margins");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), printing_prefs.custom_printer_margins);
+	on_prefs_custom_printer_margins_toggled(GTK_TOGGLE_BUTTON(widget), NULL);
+	
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_left");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), printing_prefs.margin_left);
+	
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_left_units");
+	utils_combo_box_set_units(GTK_COMBO_BOX(widget), printing_prefs.margin_left_units);
+	
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_right");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), printing_prefs.margin_right);
+	
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_right_units");
+	utils_combo_box_set_units(GTK_COMBO_BOX(widget), printing_prefs.margin_right_units);
+	
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_top");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), printing_prefs.margin_top);
+	
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_top_units");
+	utils_combo_box_set_units(GTK_COMBO_BOX(widget), printing_prefs.margin_top_units);
+	
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_bottom");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), printing_prefs.margin_bottom);
+
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_bottom_units");
+	utils_combo_box_set_units(GTK_COMBO_BOX(widget), printing_prefs.margin_bottom_units);
+
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_print_basename");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), printing_prefs.page_header_basename);
 
@@ -1140,6 +1167,33 @@
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_print_pageheader");
 		printing_prefs.print_page_header = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_custom_margins");
+		printing_prefs.custom_printer_margins = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+		
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_left");
+		printing_prefs.margin_left = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
+		
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_left_units");
+		printing_prefs.margin_left_units = utils_combo_box_get_units(GTK_COMBO_BOX(widget));
+
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_right");
+		printing_prefs.margin_right = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
+		
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_right_units");
+		printing_prefs.margin_right_units = utils_combo_box_get_units(GTK_COMBO_BOX(widget));
+
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_top");
+		printing_prefs.margin_top = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
+		
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_top_units");
+		printing_prefs.margin_top_units = utils_combo_box_get_units(GTK_COMBO_BOX(widget));
+
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_bottom");
+		printing_prefs.margin_bottom = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
+		
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_bottom_units");
+		printing_prefs.margin_bottom_units = utils_combo_box_get_units(GTK_COMBO_BOX(widget));
+
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_print_basename");
 		printing_prefs.page_header_basename = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
@@ -1532,7 +1586,24 @@
 	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_print_dateformat"), sens);
 }
 
+static void on_prefs_custom_printer_margins_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	gboolean sens = gtk_toggle_button_get_active(togglebutton);
+	
+	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_left"), sens);
+	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_left_units"), sens);
+	
+	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_right"), sens);
+	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_right_units"), sens);
 
+	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_top"), sens);
+	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_top_units"), sens);
+
+	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "spin_margin_bottom"), sens);
+	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "combo_margin_bottom_units"), sens);
+
+}
+
 static void open_preferences_help(void)
 {
 	gchar *uri;
@@ -1756,6 +1827,10 @@
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_print_pageheader"),
 			"toggled", G_CALLBACK(on_prefs_print_page_header_toggled), NULL);
 
+		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_custom_margins"),
+			"toggled", G_CALLBACK(on_prefs_custom_printer_margins_toggled), NULL);
+
+
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_plugins"),
 				"toggled", G_CALLBACK(on_enable_plugins_toggled), NULL);
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_toolbar_show"),
--- ./src/keyfile.c	(revision 5530)
+++ ./src/keyfile.c	(working copy)
@@ -444,6 +444,17 @@
 	g_key_file_set_boolean(config, "printing", "print_line_numbers", printing_prefs.print_line_numbers);
 	g_key_file_set_boolean(config, "printing", "print_page_numbers", printing_prefs.print_page_numbers);
 	g_key_file_set_boolean(config, "printing", "print_page_header", printing_prefs.print_page_header);
+	
+	g_key_file_set_boolean(config, "printing", "custom_printer_margins", printing_prefs.custom_printer_margins);
+	g_key_file_set_double(config, "printing", "margin_left", printing_prefs.margin_left);
+	g_key_file_set_integer(config, "printing", "margin_left_units", printing_prefs.margin_left_units);
+	g_key_file_set_double(config, "printing", "margin_right", printing_prefs.margin_right);
+	g_key_file_set_integer(config, "printing", "margin_right_units", printing_prefs.margin_right_units);
+	g_key_file_set_double(config, "printing", "margin_top", printing_prefs.margin_top);
+	g_key_file_set_integer(config, "printing", "margin_top_units", printing_prefs.margin_top_units);
+	g_key_file_set_double(config, "printing", "margin_bottom", printing_prefs.margin_bottom);
+	g_key_file_set_integer(config, "printing", "margin_bottom_units", printing_prefs.margin_bottom_units);
+	
 	g_key_file_set_boolean(config, "printing", "page_header_basename", printing_prefs.page_header_basename);
 	g_key_file_set_string(config, "printing", "page_header_datefmt", printing_prefs.page_header_datefmt);
 
@@ -835,6 +846,17 @@
 	printing_prefs.print_line_numbers = utils_get_setting_boolean(config, "printing", "print_line_numbers", TRUE);
 	printing_prefs.print_page_numbers = utils_get_setting_boolean(config, "printing", "print_page_numbers", TRUE);
 	printing_prefs.print_page_header = utils_get_setting_boolean(config, "printing", "print_page_header", TRUE);
+	
+	printing_prefs.custom_printer_margins = utils_get_setting_boolean(config, "printing", "custom_printer_margins", FALSE);
+	printing_prefs.margin_left = utils_get_setting_double(config, "printing", "margin_left", 0);
+	printing_prefs.margin_left_units = utils_get_setting_integer(config, "printing", "margin_left_units", GTK_UNIT_MM);
+	printing_prefs.margin_right = utils_get_setting_double(config, "printing", "margin_right", 0);
+	printing_prefs.margin_right_units = utils_get_setting_integer(config, "printing", "margin_right_units", GTK_UNIT_MM);
+	printing_prefs.margin_top = utils_get_setting_double(config, "printing", "margin_top", 0);
+	printing_prefs.margin_top_units = utils_get_setting_integer(config, "printing", "margin_top_units", GTK_UNIT_MM);
+	printing_prefs.margin_bottom = utils_get_setting_double(config, "printing", "margin_bottom", 0);
+	printing_prefs.margin_bottom_units = utils_get_setting_integer(config, "printing", "margin_bottom_units", GTK_UNIT_MM);
+	
 	printing_prefs.page_header_basename = utils_get_setting_boolean(config, "printing", "page_header_basename", FALSE);
 	printing_prefs.page_header_datefmt = utils_get_setting_string(config, "printing", "page_header_datefmt", "%c");
 }
--- ./geany.glade	(revision 5530)
+++ ./geany.glade	(working copy)
@@ -9906,6 +9906,350 @@
 				      <property name="fill">True</property>
 				    </packing>
 				  </child>
+
+				  <child>
+				    <widget class="GtkCheckButton" id="check_custom_margins">
+				      <property name="visible">True</property>
+				      <property name="tooltip" translatable="yes">Use custom printer margins, overriding printer settings</property>
+				      <property name="can_focus">True</property>
+				      <property name="label" translatable="yes">Use custom printer margins</property>
+				      <property name="use_underline">True</property>
+				      <property name="relief">GTK_RELIEF_NORMAL</property>
+				      <property name="focus_on_click">True</property>
+				      <property name="active">False</property>
+				      <property name="inconsistent">False</property>
+				      <property name="draw_indicator">True</property>
+				    </widget>
+				    <packing>
+				      <property name="padding">0</property>
+				      <property name="expand">False</property>
+				      <property name="fill">False</property>
+				    </packing>
+				  </child>
+
+				  <child>
+				    <widget class="GtkFrame" id="frame1">
+				      <property name="visible">True</property>
+				      <property name="label_xalign">0</property>
+				      <property name="label_yalign">0</property>
+				      <property name="shadow_type">GTK_SHADOW_NONE</property>
+
+				      <child>
+					<widget class="GtkAlignment" id="alignment1">
+					  <property name="visible">True</property>
+					  <property name="xalign">0</property>
+					  <property name="yalign">0.5</property>
+					  <property name="xscale">1</property>
+					  <property name="yscale">1</property>
+					  <property name="top_padding">0</property>
+					  <property name="bottom_padding">0</property>
+					  <property name="left_padding">12</property>
+					  <property name="right_padding">0</property>
+
+					  <child>
+					    <widget class="GtkHBox" id="hbox1">
+					      <property name="visible">True</property>
+					      <property name="homogeneous">False</property>
+					      <property name="spacing">10</property>
+
+					      <child>
+						<widget class="GtkTable" id="table3">
+						  <property name="visible">True</property>
+						  <property name="n_rows">2</property>
+						  <property name="n_columns">3</property>
+						  <property name="homogeneous">False</property>
+						  <property name="row_spacing">0</property>
+						  <property name="column_spacing">5</property>
+
+						  <child>
+						    <widget class="GtkLabel" id="label1">
+						      <property name="visible">True</property>
+						      <property name="label" translatable="yes">Left:</property>
+						      <property name="use_underline">False</property>
+						      <property name="use_markup">False</property>
+						      <property name="justify">GTK_JUSTIFY_LEFT</property>
+						      <property name="wrap">False</property>
+						      <property name="selectable">False</property>
+						      <property name="xalign">1</property>
+						      <property name="yalign">0.5</property>
+						      <property name="xpad">0</property>
+						      <property name="ypad">0</property>
+						      <property name="ellipsize">PANGO_ELLIPSIZE_NONE</property>
+						      <property name="width_chars">-1</property>
+						      <property name="single_line_mode">False</property>
+						      <property name="angle">0</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">0</property>
+						      <property name="right_attach">1</property>
+						      <property name="top_attach">0</property>
+						      <property name="bottom_attach">1</property>
+						    </packing>
+						  </child>
+
+						  <child>
+						    <widget class="GtkLabel" id="label2">
+						      <property name="visible">True</property>
+						      <property name="label" translatable="yes">Right:</property>
+						      <property name="use_underline">False</property>
+						      <property name="use_markup">False</property>
+						      <property name="justify">GTK_JUSTIFY_LEFT</property>
+						      <property name="wrap">False</property>
+						      <property name="selectable">False</property>
+						      <property name="xalign">1</property>
+						      <property name="yalign">0.5</property>
+						      <property name="xpad">0</property>
+						      <property name="ypad">0</property>
+						      <property name="ellipsize">PANGO_ELLIPSIZE_NONE</property>
+						      <property name="width_chars">-1</property>
+						      <property name="single_line_mode">False</property>
+						      <property name="angle">0</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">0</property>
+						      <property name="right_attach">1</property>
+						      <property name="top_attach">1</property>
+						      <property name="bottom_attach">2</property>
+						    </packing>
+						  </child>
+
+						  <child>
+						    <widget class="GtkSpinButton" id="spin_margin_left">
+						      <property name="visible">True</property>
+						      <property name="can_focus">True</property>
+						      <property name="climb_rate">1</property>
+						      <property name="digits">0</property>
+						      <property name="numeric">False</property>
+						      <property name="update_policy">GTK_UPDATE_ALWAYS</property>
+						      <property name="snap_to_ticks">False</property>
+						      <property name="wrap">False</property>
+						      <property name="adjustment">10 0 100 1 10 10</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">1</property>
+						      <property name="right_attach">2</property>
+						      <property name="top_attach">0</property>
+						      <property name="bottom_attach">1</property>
+						    </packing>
+						  </child>
+
+						  <child>
+						    <widget class="GtkComboBox" id="combo_margin_left_units">
+						      <property name="visible">True</property>
+						      <property name="items" translatable="yes">px
+pt
+inch
+mm</property>
+						      <property name="add_tearoffs">False</property>
+						      <property name="focus_on_click">True</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">2</property>
+						      <property name="right_attach">3</property>
+						      <property name="top_attach">0</property>
+						      <property name="bottom_attach">1</property>
+						    </packing>
+						  </child>
+
+						  <child>
+						    <widget class="GtkSpinButton" id="spin_margin_right">
+						      <property name="visible">True</property>
+						      <property name="can_focus">True</property>
+						      <property name="climb_rate">1</property>
+						      <property name="digits">0</property>
+						      <property name="numeric">False</property>
+						      <property name="update_policy">GTK_UPDATE_ALWAYS</property>
+						      <property name="snap_to_ticks">False</property>
+						      <property name="wrap">False</property>
+						      <property name="adjustment">10 0 100 1 10 10</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">1</property>
+						      <property name="right_attach">2</property>
+						      <property name="top_attach">1</property>
+						      <property name="bottom_attach">2</property>
+						    </packing>
+						  </child>
+
+						  <child>
+						    <widget class="GtkComboBox" id="combo_margin_right_units">
+						      <property name="visible">True</property>
+						      <property name="items" translatable="yes">px
+pt
+inch
+mm</property>
+						      <property name="add_tearoffs">False</property>
+						      <property name="focus_on_click">True</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">2</property>
+						      <property name="right_attach">3</property>
+						      <property name="top_attach">1</property>
+						      <property name="bottom_attach">2</property>
+						    </packing>
+						  </child>
+						</widget>
+						<packing>
+						  <property name="padding">0</property>
+						  <property name="expand">False</property>
+						  <property name="fill">False</property>
+						</packing>
+					      </child>
+
+					      <child>
+						<widget class="GtkTable" id="table4">
+						  <property name="visible">True</property>
+						  <property name="n_rows">2</property>
+						  <property name="n_columns">3</property>
+						  <property name="homogeneous">False</property>
+						  <property name="row_spacing">0</property>
+						  <property name="column_spacing">5</property>
+
+						  <child>
+						    <widget class="GtkLabel" id="label3">
+						      <property name="visible">True</property>
+						      <property name="label" translatable="yes">Top:</property>
+						      <property name="use_underline">False</property>
+						      <property name="use_markup">False</property>
+						      <property name="justify">GTK_JUSTIFY_LEFT</property>
+						      <property name="wrap">False</property>
+						      <property name="selectable">False</property>
+						      <property name="xalign">1</property>
+						      <property name="yalign">0.5</property>
+						      <property name="xpad">0</property>
+						      <property name="ypad">0</property>
+						      <property name="ellipsize">PANGO_ELLIPSIZE_NONE</property>
+						      <property name="width_chars">-1</property>
+						      <property name="single_line_mode">False</property>
+						      <property name="angle">0</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">0</property>
+						      <property name="right_attach">1</property>
+						      <property name="top_attach">0</property>
+						      <property name="bottom_attach">1</property>
+						    </packing>
+						  </child>
+
+						  <child>
+						    <widget class="GtkLabel" id="label4">
+						      <property name="visible">True</property>
+						      <property name="label" translatable="yes">Bottom:</property>
+						      <property name="use_underline">False</property>
+						      <property name="use_markup">False</property>
+						      <property name="justify">GTK_JUSTIFY_LEFT</property>
+						      <property name="wrap">False</property>
+						      <property name="selectable">False</property>
+						      <property name="xalign">1</property>
+						      <property name="yalign">0.5</property>
+						      <property name="xpad">0</property>
+						      <property name="ypad">0</property>
+						      <property name="ellipsize">PANGO_ELLIPSIZE_NONE</property>
+						      <property name="width_chars">-1</property>
+						      <property name="single_line_mode">False</property>
+						      <property name="angle">0</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">0</property>
+						      <property name="right_attach">1</property>
+						      <property name="top_attach">1</property>
+						      <property name="bottom_attach">2</property>
+						    </packing>
+						  </child>
+
+						  <child>
+						    <widget class="GtkSpinButton" id="spin_margin_top">
+						      <property name="visible">True</property>
+						      <property name="can_focus">True</property>
+						      <property name="climb_rate">1</property>
+						      <property name="digits">0</property>
+						      <property name="numeric">False</property>
+						      <property name="update_policy">GTK_UPDATE_ALWAYS</property>
+						      <property name="snap_to_ticks">False</property>
+						      <property name="wrap">False</property>
+						      <property name="adjustment">10 0 100 1 10 10</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">1</property>
+						      <property name="right_attach">2</property>
+						      <property name="top_attach">0</property>
+						      <property name="bottom_attach">1</property>
+						    </packing>
+						  </child>
+
+						  <child>
+						    <widget class="GtkComboBox" id="combo_margin_top_units">
+						      <property name="visible">True</property>
+						      <property name="items" translatable="yes">px
+pt
+inch
+mm</property>
+						      <property name="add_tearoffs">False</property>
+						      <property name="focus_on_click">True</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">2</property>
+						      <property name="right_attach">3</property>
+						      <property name="top_attach">0</property>
+						      <property name="bottom_attach">1</property>
+						    </packing>
+						  </child>
+
+						  <child>
+						    <widget class="GtkSpinButton" id="spin_margin_bottom">
+						      <property name="visible">True</property>
+						      <property name="can_focus">True</property>
+						      <property name="climb_rate">1</property>
+						      <property name="digits">0</property>
+						      <property name="numeric">False</property>
+						      <property name="update_policy">GTK_UPDATE_ALWAYS</property>
+						      <property name="snap_to_ticks">False</property>
+						      <property name="wrap">False</property>
+						      <property name="adjustment">10 0 100 1 10 10</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">1</property>
+						      <property name="right_attach">2</property>
+						      <property name="top_attach">1</property>
+						      <property name="bottom_attach">2</property>
+						    </packing>
+						  </child>
+
+						  <child>
+						    <widget class="GtkComboBox" id="combo_margin_bottom_units">
+						      <property name="visible">True</property>
+						      <property name="items" translatable="yes">px
+pt
+inch
+mm</property>
+						      <property name="add_tearoffs">False</property>
+						      <property name="focus_on_click">True</property>
+						    </widget>
+						    <packing>
+						      <property name="left_attach">2</property>
+						      <property name="right_attach">3</property>
+						      <property name="top_attach">1</property>
+						      <property name="bottom_attach">2</property>
+						    </packing>
+						  </child>
+						</widget>
+						<packing>
+						  <property name="padding">0</property>
+						  <property name="expand">False</property>
+						  <property name="fill">False</property>
+						</packing>
+					      </child>
+					    </widget>
+					  </child>
+					</widget>
+				      </child>
+				    </widget>
+				    <packing>
+				      <property name="padding">0</property>
+				      <property name="expand">False</property>
+				      <property name="fill">False</property>
+				    </packing>
+				  </child>
 				</widget>
 			      </child>
 			    </widget>
